package chat;

import java.util.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;

import limone.*;
import eBeacon.*;
import eLights.*;

/**
 * The is the main class that controls everything.
 *
 * @author Chien-Liang Fok
 * @version 5/13/2003
 */
public class ChatUser implements java.io.Serializable,
	AcquaintanceChangedListener {
	
    /**
	 * The name of the user.
	 */
    String name;
	
    /**
	 * Maps the other person's agentID to the appropriate IM GUI
	 */
    Hashtable imWindows = new Hashtable();
	
    /**
	 * Used to determine when this agent's thread can run to completion
	 */
    boolean done = false;
	
	/**
	 * The GUI for this chat user.
	 */
    ChatUserGUI gui;
	
	/**
	 * The agent used for communication.
	 */
    private ChatUserAgent agent;
	
    public ChatUser(String name, ChatUserAgent agent) {
		this.name = name;
		this.agent = agent;
		gui = new ChatUserGUI(this);
    }
	
	/**
	 * This is called by the Limone middleware when an agent is added or
	 * removed from the acquaintance list.
	 */
	public void acuaintanceChanged(AcquaintanceChangedEvent ace) {
		
		Profile p = ace.getProfile();
		AgentID aID = (AgentID)(p.getProperty("_AgentID").getValue());
		InstantMessagingGUI imGUI = (InstantMessagingGUI)imWindows.get(aID);
		
		if (ace.getType() == AcquaintanceChangedEvent.AGENT_ADDED) {
			gui.userEntered(new UserInfo(aID));
			if (imGUI != null)
				imGUI.agentArrived();
		} else {
			gui.userLeft(new UserInfo(aID));
			if (imGUI != null)
				imGUI.agentLeft();
		}
	}
	
	
	
    /**
	 * Called when an instant message has arrived from a remote agent.
	 *
	 * @param aID the AgentID of the agent that sent a message.
	 * @param message the message sent from the other agent.
	 */
    void processIM(AgentID aID, String message) {
		
		log("Received an instant message from " + aID);
		// get the IM GUI
		InstantMessagingGUI imGUI = getIMRoom(aID);
		
		// add the received text to the GUI
		imGUI.addRecievedText(aID.getName(), message);
    }
    
    /**
	 * Returns the name of this chat user.
	 *
	 * @return the name of this chat user.
	 */
    public String getName() {
		return name;
    }
	
    /**
	 * Start an instant messaging session with a user at the given AgentLocation.
	 *
	 * @param aID the AgentID of the destination agent.
	 */
    public synchronized void startIM(AgentID aID) {
		InstantMessagingGUI imGUI = getIMRoom(aID);
		imGUI.toFront();
    }
	
    /**
	 * Determines the equality of another class with this one.
	 */
    public boolean equals(Object o){
		if (o instanceof ChatUser)
			return ((ChatUser)o).getName().equals(name);
		else
			return false;
    }
	
    /**
	 * Called when we want to close all IM Windows.
	 */
    private void disposeAllIMWindows(){
		Enumeration enum = imWindows.elements();
		
		while(enum.hasMoreElements()){
			InstantMessagingGUI imGUI = (InstantMessagingGUI)enum.nextElement();
			imGUI.dispose();
		}
    }
	
	/**
	 * Called when the user wishes to shutdown the agent.
	 */
    public void kill() {
		LimoneServer.getServer().killAgent(agent.getAgentID());
		disposeAllIMWindows();
		int response = JOptionPane.showConfirmDialog(
			null,
			"Shutdown the local LimoneServer?",
			"Question",
			JOptionPane.YES_NO_OPTION);
		if (response == JOptionPane.YES_OPTION)
			LimoneServer.getServer().shutdown();
    }
	
    /**
	 * Creates an InstantMessagingGUI for instant messaging another person.
	 *
	 * @param aID the AgentID of the destination agent
	 * @return the InstantMessagingGUI used to speak with the other person
	 */
    public synchronized InstantMessagingGUI getIMRoom(final AgentID aID){
		InstantMessagingGUI imGUI = (InstantMessagingGUI)imWindows.get(aID);
		if (imGUI == null){
			imGUI = new InstantMessagingGUI(this, aID);
			imGUI.addWindowListener(new WindowAdapter(){
						public void windowClosing(WindowEvent e){
							imWindows.remove(aID);
						}
					});
			// add the IM window to the hashtable, hashed by AgentID
			imWindows.put(aID, imGUI);
			
		}
		imGUI.show();
		return imGUI;
    }
	
    /**
	 * Sends a message containing the agent's AgentID
	 * to the agent located at aLoc.
	 *
	 * @param destaID the destination agent's the AgentID.
	 * @return this agent's AgentID.
	 */
    public AgentID sendIMAgentID(AgentID destaID) {
		AgentID aID = agent.getAgentID();
		sendIM(destaID, aID.toString());
		return aID;
    }
	
    /**
	 * Sends an instant message to the agent located at
	 * location aLoc.
	 *
	 * @param aID the destination agent's AgentID.
	 * @param msg the message to be sent.
	 */
    public synchronized void sendIM(AgentID aID, String msg) {
		agent.sendIM(aID, msg);
    }
	
    private void log(String msg) {
		System.out.println("ChatUser: " + msg);
    }
}
