package chat;

import limone.*;
import eLights.*;
import eBeacon.*;
import java.util.*;

/**
 * Handles all of the underlying communication between chat participants.
 *
 * @author Chien-Liang Fok
 * @version 5/13/2003
 */
public class ChatUserAgent extends Agent implements ReactionListener {
	
	public static final double VERSION_NUMBER = 1.0;
	
	/**
	 * A reference to the ChatUser, which interfaces between the GUI
	 * and this agent.
	 */
    ChatUser user;
	
    /**
	 * Create a UserAgent object.
	 *
	 * @param ChatUser the user using this object.
	 * @param AgentID the AgentID of this agent.
	 */
    public ChatUserAgent(AgentID aID, String name) {
		super(aID, name);
		user = new ChatUser(name, this);
		aql.addChangeListener(user);
		
		log("ChatUserAgent created.");
    }
	
    /**
	 * Sends an instant message by creating a special message tuple.
	 * The message tuple contains a single field called "msg" with
	 * a value of type string consisting of the message being sent.
	 *
	 * @param aID the destination agent's AgentID.
	 * @param msg the message to be sent to the destination agent.
	 */
    public void sendIM(AgentID aID, String msg) {
		ETuple tuple = new ETuple();
		tuple.addField(new EField("msg", msg));
		tuple.addField(new EField("aID", getAgentID()));
		
		AgentProxy ap = aql.getAgentProxy(aID);
		
		if (ap != null) {
			try {
				ap.out(tuple);
			} catch(RejectionException re) {
				re.printStackTrace();
			} catch (TimeOutException toe) {
				toe.printStackTrace();
			}
		} else {
			// display error
		}
		
    }
	
    /**
	 * Implements the ReactionListener interface.  Since only one reaction
	 * is registered, anytime this method is called, a new message tuple
	 * must have been found.
	 */
    public void reactsTo(ReactionEvent re) {
				
		ETuple tuple = re.tuple();
		
		// extract the data out of the tuple
		try {
			AgentID aID = (AgentID)tuple.getField("aID").getValue();
			String msg = (String)tuple.getField("msg").getValue();
			user.processIM(aID, msg);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Initializes the agent by registering a reaction sensitive to messages
	 * on the local tuple space.
	 */
	public void agentInit() {
		
		// Create the template for message tuples destined for this agent.
		ETemplate template = new ETemplate();
		template.addConstraint(
			new EConstraint("msg", String.class,
							new DefaultConstraintFunction()));
		template.addConstraint(
			new EConstraint("aID", AgentID.class,
							new DefaultConstraintFunction()));
		
		// Create a profile selector that limits the propagation of this
		// reaction to the local tuple space.
		ProfileSelector ps = new ProfileSelector();
		ps.addConstraint(
			new ProfileConstraint(
							"_AgentID", AgentID.class,
							new EquivalencyConstraintFunction(getAgentID())));
		
		// Create the reactive pattern.
		ReactivePattern rPat = new ReactivePattern(
			ps, Reaction.ONCE_PER_TUPLE, template);
		
		// Create the reaction
		Reaction reaction = new Reaction(rPat, this);
		
		log("Registering the message reaction.");
		rr.registerReaction(reaction);
	}
	
	/**
	 *
	 */
	public void agentRun() {
	}
	
	/**
	 *
	 */
	public void agentDone() {
	}
	
	/**
	 * Called when the LimoneServer on which this agent is
	 * residing is about to shut down.
	 */
	public void shutDown() {
	}
	
	/**
	 * Debug printouts
	 */
	private void log(String msg) {
		System.out.println("UserAgent: " + msg);
	}
}
