package chat;

import java.util.*;

/**
 * An implementation of a branch within the tree
 *
 * @author Chien-Liang Fok
 * @version 1/2/2002
 */
public class TreeBranch implements java.io.Serializable{
	/**
	  * The name of the branch
	  */
	private String category;	

	/**
	  * contains TreeLeaf objects
	  */
	private Vector leaves = new Vector();
	
	/**
	 * Constructs a tree branch of name category
	 * 
	 * @param category the category of the branch
	 */
	public TreeBranch(String cat){
		this.category = cat;
	}
	
	/**
	 * Returns the number of leaves.
	 *
	 * @return the number of leaves in this branch.
	 */
	public synchronized int getNumLeaves(){
		return leaves.size();
	}
	
	/**
	 * Returns the category (name) of this branch.
	 *
	 * @return the category (name) of this branch
	 */
	public String getCategory(){
		return category;
	}
	
	/**
	 * Returns the child at the index send in.
	 *
	 * @param index the index of the child we are looking for
	 * @return the child of index index
	 */
	public synchronized TreeLeaf getChild(int index){
		if (index >= leaves.size())
			return null;
		else
			return (TreeLeaf)leaves.get(index);
	}
	
	/**
	 * Add a new child to this branch.  If the child already 
	 * exists, update it.  Adding is successful if the leaf
	 * did not already exist.
	 *
	 * @return true if successful
	 */
	public synchronized boolean addLeaf(UserInfo info, TreeChangeResults results){
		TreeLeaf dummy = new TreeLeaf(info);
		if (leaves.contains(dummy)) {
			log("addLeaf - The leaf already existed, cannot add, returning false");
			return false;
		} 
		else {
			log("addLeaf - the leaf did not already exist");
			TreeLeaf leaf = new TreeLeaf(info);
			leaves.add(leaf);
			results.addNode(leaves.indexOf(leaf), leaf);
			log("addLeaf - done addLeaf, successful");
			return true;
		}
	}

	/**
	 * Update a child in this branch.  If the child already 
	 * exists, update it.
	 *
	 * @return true if the operation was successful
	 */
	public synchronized boolean updateLeaf(UserInfo info, TreeChangeResults results){
		TreeLeaf dummy = new TreeLeaf(info);
		if (leaves.contains(dummy)) {
			log("addLeaf - The leaf already existed, updating");
			int index = leaves.indexOf(dummy);
			TreeLeaf leaf = (TreeLeaf)leaves.get(index);
			leaf.setUserInfo(info);
			results.addNode(index, leaf);
			log("updateLeaf - operation successful");
			return true;
		} 
		else {
			log("updateLeaf - the leaf did not already exist");
			return false;
		}
	}
	
	/**
	 * Removes the child with a particular name
	 *
	 * @param name the name of the child we are removing	
	 * @return true if successful
	 */
	public synchronized boolean removeLeaf(UserInfo info, TreeChangeResults results){
		TreeLeaf dummy = new TreeLeaf(info);
		if (leaves.contains(dummy)) {
			int index = leaves.indexOf(dummy);
			results.addNode(index, leaves.get(index));
			leaves.removeElementAt(index);
			log("successfully removed leaf " + info);
			return true;
		}
		else{
			log("removeLeaf: The leaf was not found");
			return false;
		}
	}

	/**
	 * Returns the index of the child sent in, returns -1 if
	 * the child is not in this branch.
	 *
	 * @param child the child whos index we are looking for
	 * @return the index of the child, or -1 if the child is not present
	 */
	public synchronized int getIndexOfChild(Object child){
		return leaves.indexOf(child);
	}
	
	/**
	 * Determines if an object is equal to this one
	 * An object is equal to this object if and only if
	 * the object is an instance of TreeBranch and
	 * the categories of the two are the same.  Or, the
	 * object is a string that matches the category.
	 *
	 * @param o the object to be compared with this one
	 * @return true if this branch is equal to cutb
	 */
	public boolean equals(Object o){
		//log("comparing equals to " + o);
		if (o == null)
			return false;		
		if (o instanceof TreeBranch){
			//log("instanceof treebranch");
			TreeBranch branch = (TreeBranch) o;
			return branch.getCategory().equals(this.category);
		}

		if (o instanceof String) {
			//log("instanceof string");
			String cat = (String)o;
			return cat.equals(category);
		}
		
		return false;
	}
	
	private void log(String str){
		//System.out.println("TreeBranch: " + str);
		//System.out.flush();
	}
	
	/**
	 * The String representation of this class.
	 *
	 * @return the string representation of this class.
	 */
	public String toString(){
		return category;
	}
}
