package chat;

import javax.swing.event.*;
import java.util.*;

/**
 * The model of the tree used in ChatUser
 *
 * @author Chien-Liang Fok
 * @version 1/2/2002
 */
public class TreeModel implements javax.swing.tree.TreeModel, java.io.Serializable{
	/**
	  * A vector of treeListeners
	  */
	Vector treeListenerVector = new Vector();	

	/**
	  * The root of the tree
	  */
	TreeRoot root = new TreeRoot();
	
	/**
	 * Constructs a tree model
	 */
	public TreeModel(){	
	}

	/**
	  * Adds a listener for the TreeModelEvent posted after the tree changes.
	  * Implements the TreeModel interface.
	  */
	public void addTreeModelListener(TreeModelListener l){
		treeListenerVector.add(l);
	}

	/**
	  * Removes a listener previously added with addTreeModelListener().
  	  * Implements the TreeModel interface.
	  */
	public void removeTreeModelListener(TreeModelListener l){
		treeListenerVector.remove(l);
	}

	/**
	  * Returns the child of parent at index index in the parent's child array. 
	  * parent must be a node previously obtained from this data source. 
	  * This should not return null if index is a valid index for parent 
	  * (that is index >= 0 && index < getChildCount(parent)).
	  * Implements the TreeModel interface.
	  *
	  * @param parent a node in the tree, obtained from this data source
	  * @return the child of parent at index index
	  */
	public Object getChild(Object parent, int index){
		if (parent instanceof TreeRoot)
			return ((TreeRoot)parent).getChild(index);
		else if (parent instanceof TreeBranch)
			return ((TreeBranch)parent).getChild(index);
		else
			return null;
	}

	/**
	  *  Implements the TreeModel interface.
	  */
	public int getChildCount(Object parent){
		if (parent instanceof TreeRoot)
			return ((TreeRoot)parent).getNumBranches();
		else if (parent instanceof TreeBranch)
			return ((TreeBranch)parent).getNumLeaves();
		else
			return 0;
	}

	/**
	  *  Implements the TreeModel interface.
	  */
	public int getIndexOfChild(Object parent, Object child){
		if (parent instanceof TreeRoot)
			return ((TreeRoot)parent).getIndexOfChild(child);
		else if (parent instanceof TreeBranch)
			return ((TreeBranch)parent).getIndexOfChild(child);
		else
			return -1;
	}

	/**
	  *  Implements the TreeModel interface.
	  */
	public Object getRoot(){
		return root;
	}

	/**
	  *  Implements the TreeModel interface.
	  */
	public boolean isLeaf(Object node){
		return (node instanceof TreeLeaf);
	}

	/**
	  *  Implements the TreeModel interface.
	  */
	public void valueForPathChanged(javax.swing.tree.TreePath path, Object newValue) {
	}

	/**
	  * Updates a leaf in the tree.
	  *
	  * @return true if update successful
	  */
	public synchronized boolean updateLeaf(String category, UserInfo info){
		log("updateLeaf called, category = " + category);
		TreeChangeResults results = new TreeChangeResults(); 
		results.addToPath(root);
		if (root.updateLeaf(category, info, results)) {

			log(" results.getIndices() = " +  arrayToString(results.getIndices()));
			log(" results.getNodes() = " +  arrayToString(results.getNodes()));
			log(" results.getPath() = " +  arrayToString(results.getPath()));
		
			notifytreeNodesChanged(new TreeModelEvent(this, results.getPath(), results.getIndices(), results.getNodes()));		
			log("Done update leaf");
			return true;
		}
		else {
			log("Error updating the leaf " + category + "/" + info);
			return false;
		}
	}
	
	private String arrayToString(int[] o) {
		String result = "";
		for (int i = 0; i < o.length; i++) {
			result += o[i]+ " ";
		}
		return result;
	}

	private String arrayToString(Object[] o) {
		String result = "";
		for (int i = 0; i < o.length; i++) {
			result += o[i].toString() + " ";
		}
		return result;
	}
	
	/**
	 * This method is called whenever new files are available.
	 * @param sharedFilesVector a vector of TreeLeaf objects
	 * @return true if successful
	 */
	public synchronized boolean addLeaf(String category, UserInfo info){
		TreeChangeResults results = new TreeChangeResults();
		results.addToPath(root);
		
		if (root.addLeaf(category, info, results)){
			notifyTreeNodesInserted(new TreeModelEvent(this, results.getPath(), results.getIndices(), results.getNodes()));
			log("add leaf successful");
			return true;
		}
		else {
			log("addLeaf unsuccessful");
			return false;
		}
	}

	/**
	 * Removes the leaf of a particular category and UserInfo.
	 *
	 * @param category the branch category of the leaf to be removed
	 * @param name the name of the leaf to be removed
	 * @return true if successful
	 */
	public synchronized boolean removeLeaf(String category, UserInfo info){
		log("removeLeaf called: " + category + ", " + info);
		TreeChangeResults results = new TreeChangeResults();
		results.addToPath(root);

		if (root.removeLeaf(category, info, results)) {
			log("removeLeaf: notifying tree nodes removed");

			log(" results.getIndices() = " +  arrayToString(results.getIndices()));
			log(" results.getNodes() = " +  arrayToString(results.getNodes()));			
			log(" results.getPath() = " +  arrayToString(results.getPath()));
			notifyTreeNodesRemoved(new TreeModelEvent(this, results.getPath(), results.getIndices(), results.getNodes()));	
			log("removeLeaf: end removeLeaf");			
			return true;
		}
		else {
			log("removeLeaf unsuccessful");
			return false;
		}
	}
	
	/**
	 * Returns the branch of the given category, or null if category does not exist
	 *
	 * @param category the branch we are looking for
	 * @return the branch of a particular category
	 */
	 /*
	public synchronized TreeBranch getBranch(String category){
		return root.getChild(category);
	}
	*/
	/**
	 * Returns the tree leaf at the end of the path
	 *
	 * @param selPath the TreePath to the leaf
	 * @return the leaf lead to by the path
	 */
	public synchronized TreeLeaf getTreeLeaf(javax.swing.tree.TreePath selPath){
		return (TreeLeaf)selPath.getPathComponent(2);
	}
	
	/**
	 * Called when there is a major change to the tree
	 */
	private synchronized void notifyTreeStrucChange(TreeModelEvent tme){
		for (int i = 0; i < treeListenerVector.size(); i++){
			((TreeModelListener)treeListenerVector.get(i))
				.treeStructureChanged(tme);
		}
	}
	
	/**
	 * Called when a node has changed
	 */
	private synchronized void notifytreeNodesChanged(TreeModelEvent tme){
		for (int i = 0; i < treeListenerVector.size(); i++){
			((TreeModelListener)treeListenerVector.get(i))
				.treeNodesChanged(tme);
		}
	}
	
	/**
	 * Called when a node is removed from the tree.
	 */
	private synchronized void notifyTreeNodesRemoved(TreeModelEvent tme){
		for (int i = 0; i < treeListenerVector.size(); i++){
			((TreeModelListener)treeListenerVector.get(i))
				.treeNodesRemoved(tme);
		}
	}
	
	/**
	 * Called when a node has been inserted.
	 */
	private synchronized void notifyTreeNodesInserted(TreeModelEvent tme){
		for (int i = 0; i < treeListenerVector.size(); i++){
			((TreeModelListener)treeListenerVector.get(i))
				.treeNodesInserted(tme);
		}
	}
	
	private void log(String str){
		//System.out.println("TreeModel: " + str);
		//System.out.flush();
	}
}
